/*
 * Copyright (c) 2002 Ralf Habacker 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id:$
 */

#include <iostream>
#include <sstream>

#include "objectfile.h"


// lists Imagebase and Image size of a dll 

BOOL GetImageInfos(char *filename, uint *ImageBase, uint *ImageSize) 
{
	LinkedObjectFile dll(filename); 

	cerr << filename << ": "; 

	if (!dll.isLoaded()) {
		cerr << "error: could not open file" << endl; 
		return false; 
	}
	
	*ImageBase = dll.getNTHeader()->OptionalHeader.ImageBase;
	*ImageSize = dll.getNTHeader()->OptionalHeader.SizeOfImage;

	cerr << "ImageBase: 0x" << hex << *ImageBase << " ImageSize: 0x" << *ImageSize << endl;
	return true; 
}


BOOL CheckImage(char *filename) 
{
	LinkedObjectFile dll(filename); 
	
	cerr << filename << ": " << endl; 

	if (!dll.isLoaded()) {
		cerr << "error: could not open file" << endl; 
		return false; 
	}
	
	if (dll.checkRelocations());
}

BOOL FixImage(char *filename) 
{
	LinkedObjectFile dll(filename,true); 

	cerr << filename << ": " << endl; 

	if (!dll.isLoaded()) {
		cerr << "error: could not open file" << endl; 
		return false; 
	}
	
	return dll.fixRelocations();
}



BOOL ReBaseImage(
  PSTR CurrentImageName,  
  PSTR SymbolPath,      		// ignored 
  BOOL fReBase,          		
  BOOL fRebaseSysfileOk,  	// ignored 
  BOOL fGoingDown,       		
  ULONG CheckImageSize,  		// ignored 
  ULONG *OldImageSize,  
  ULONG *OldImageBase,  
  ULONG *NewImageSize,  
  ULONG *NewImageBase,  
  ULONG TimeStamp        
)
{
	if (fReBase == 0) {
		SetLastError(ERROR_INVALID_PARAMETER);
		return false; 
	}

	LinkedObjectFile dll(CurrentImageName,true); 


	if (!dll.checkRelocations()) {
		cerr << "dll not relocated - please fix the errors at first" << endl;
		return false; 	
	}
	
	PIMAGE_NT_HEADERS ntheader = dll.getNTHeader(); 
	
	// set new header elements 
	*OldImageBase = ntheader->OptionalHeader.ImageBase; 
	*OldImageSize = ntheader->OptionalHeader.SizeOfImage; 
	*NewImageSize = ntheader->OptionalHeader.SizeOfImage; 

	if (fGoingDown)
		*NewImageBase -= *NewImageSize; 
		
	// already rebased 
	if (ntheader->OptionalHeader.ImageBase == *NewImageBase) {
		cerr << "dll is already rebased" << endl;
		SetLastError(0);
		return true; 
	}

	ntheader->OptionalHeader.ImageBase = *NewImageBase;
	ntheader->FileHeader.TimeDateStamp = TimeStamp; 

	int difference = *NewImageBase - *OldImageBase; 

	dll.performRelocation(difference);

	SetLastError(0);
	return true; 
}
