/*
 * Copyright (c) 2002 Ralf Habacker  <Ralf.Habacker@freenet.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id:$
 */

#ifndef SECTIONS_H
#define SECTIONS_H

#include <windows.h>

typedef unsigned int uint;
typedef unsigned short ushort;

class Base {
	public: 
		static int debug; 
};

class SectionBase : public Base {
	public: 
		int getAdjust() { return adjust; }

	protected:
		int adjust; 		
	private: 
};

class Section : public SectionBase {
	public: 
		Section(void *FileBase, PIMAGE_SECTION_HEADER p);
		void print(char *title = ""); 
		bool isIn(Section &in);
		bool isIn(uint addr); 
		char *getName(void)           { return Name; } 
		int getVirtualAddress(void)   { return header->VirtualAddress; } 
		int getSize(void)             { return header->SizeOfRawData; } 
		
		// return memory adress of section 
		void *getStartAddress(void)   { return (void *)(header->VirtualAddress + adjust); } 

		void *rva2real(uint addr = 0) { return (void *)(addr + adjust); } 
	private:
		char Name[9]; 
//		uint FileBase; 
		PIMAGE_SECTION_HEADER header; 
}; 

#define SECTIONLIST_MAXSECTIONS 50

class SectionList : public Base {
	public: 
		SectionList(void *FileBase); 
		~SectionList(); 
		bool add(Section *asection);
		Section *find(char *name);
		Section *find(uint address); 

		// reset iterator 
		void reset(void);

		// return next item 
		Section *getNext(void);
		
	private: 
		uint FileBase; 
		PIMAGE_SECTION_HEADER header; 	
		Section *sections[SECTIONLIST_MAXSECTIONS]; 
		int count; 
		int iterator; 
}; 



class Exports : SectionBase {
	public: 
		Exports(Section &asection);
		Exports(SectionList &sections, PIMAGE_DATA_DIRECTORY iddp);
		uint getVirtualAddress(char *symbol, uint *ordinal = 0);
//		int getAdjust() { return adjust; }

		void reset();

		// return next exported name 
		char *getNext(void);

		void dump(char *title = "");
		
	private: 
		PIMAGE_EXPORT_DIRECTORY exports;
		PIMAGE_DATA_DIRECTORY header; 
		int iterator;
//		int adjust; 		
		
};

class ImportDescriptor : IMAGE_IMPORT_DESCRIPTOR {
};

class Imports {
	public: 
		Imports(Section &asection);
		Imports(SectionList &sections, PIMAGE_DATA_DIRECTORY iddp);
		void reset(void);
		PIMAGE_IMPORT_DESCRIPTOR getNextDescriptor(void);
		void dump(char *title = "");
		int getAdjust() { return adjust; }
		
	private: 
		PIMAGE_IMPORT_DESCRIPTOR imports; 
		PIMAGE_IMPORT_DESCRIPTOR iterator; 
		int adjust; 		
};

class Relocations : SectionBase {
	public: 
		// create a relocation object using section named "section" from  the section list "sections" 
		Relocations(SectionList &sectionList, char *sectionName);

		// check for bad relocations 
		bool check(void); 

		// fix bad relocations 
		bool fix(void); 

		// precondition: fixed dll
		bool relocate(int difference); 
		
	private:
		PIMAGE_BASE_RELOCATION relocs;
		SectionList *sections;
		int size; 		// section size 
}; 

#endif 

